unit CarRoundMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DirectDraw, DDUtil, ExtCtrls; //ddutil neni standard

type
  TCarForm = class(TForm)
    DXTimer: TTimer;
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DXTimerTimer(Sender: TObject);
  private
    { Private declarations }
    FPhase, FActive: Boolean;
    FAngle: Integer;
    FDD: IDirectDraw7;
    FDDSPrimary: IDirectDrawSurface7;
    FDDSOne: IDirectDrawSurface7;
    FDDPal: IDirectDrawPalette;
    FDDClipper: IDirectDrawClipper;
    procedure FormSetCursor(var Msg: TMessage); message WM_SETCURSOR;
    procedure ErrorOut(hRet: hResult; FuncName: string);
    function  InitDirectDraw: Boolean;
    function  RestoreAll: hResult;
    function  UpdateFrame: Boolean;
    procedure ExitDirectDraw;
  public
     { Public declarations }
  end;

var
  CarForm: TCarForm;

implementation

{$R *.DFM}

const szBitmap: PChar = 'all.bmp';

procedure TCarForm.ErrorOut(hRet: hResult; FuncName: string);
var s: string;
begin
  s := FuncName + ': ' + #13 + DDErrorString(hRet);
  MessageDlg(s, mtWarning, [mbOk], 0);
end;

procedure TCarForm.ExitDirectDraw;
begin
  if Assigned(FDD) then
  begin
    if Assigned(FDDSPrimary) then FDDSPrimary := nil;
    if Assigned(FDDSOne) then FDDSOne := nil;
    if Assigned(FDDPal) then FDDPal := nil;
    FDD := nil;
  end;
end;

procedure TCarForm.FormSetCursor(var Msg: TMessage);
begin
  SetCursor(0);
  Msg.Result := 1;
end;

function TCarForm.InitDirectDraw: Boolean;
var ddsd: TDDSurfaceDesc2;
    ddscaps: TDDSCaps2;
    hRet: hResult;
    ddpf: TDDPixelFormat;
begin
  Result := False;

  // hlavni directdraw7
  hRet := DirectDrawCreateEx(nil, FDD, IDirectDraw7, nil);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'DirectDrawCreateEx');
    Exit;
  end;

  // nastav kooperativu
  hRet := FDD.SetCooperativeLevel(Handle, DDSCL_NORMAL);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetCooperativeLevel');
    Exit;
  end;

  // vytvori prim surface
  FillChar(ddsd, SizeOf(ddsd), 0);
  ddsd.dwSize := SizeOf(ddsd);
  ddsd.dwFlags := DDSD_CAPS;
  ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;
  hRet := FDD.CreateSurface(ddsd, FDDSPrimary, nil);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'CreateSurface');
    Exit;
  end;

  // vytvorime si clipper
  hRet := FDD.CreateClipper(0, FDDClipper, nil);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'CreateClipper');
    Exit;
  end;
  hRet := FDDClipper.SetHWnd(0, Handle);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetHWnd');
    Exit;
  end;
  hRet := FDDSPrimary.SetClipper(FDDClipper);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetClipper');
    Exit;
  end;

  // zjistime barevnou hloubku
  FillChar(ddpf, SizeOf(ddpf), 0);
  ddpf.dwSize := SizeOf(ddpf);
  hRet := FDDSPrimary.GetPixelFormat(ddpf);
  if hRet <> DD_OK then
    begin
      ErrorOut(hRet, 'GetPixelFormat');
      Exit;
    end;

  // paleta bude nutna
  if ddpf.dwFlags and DDPF_PALETTEINDEXED8 = DDPF_PALETTEINDEXED8 then
  begin
    FDDPal := DDLoadPalette(FDD, szBitmap);
    if FDDPal <> nil then
    begin
      hRet := FDDSPrimary.SetPalette(FDDPal);
      if hRet <> DD_OK then
      begin
        ErrorOut(hRet, 'SetPalette');
        Exit;
      end;
    end else begin
      ErrorOut(hRet, 'DDLoadPalette');
      Exit;
    end;
  end;

  // vytvori offscreenove surface na nahrani nasi bitmapy
  FDDSOne := DDLoadBitmap(FDD, szBitmap, 0, 0);
  if FDDSOne = nil then
    begin
      ErrorOut(hRet, 'DDLoadBitmap');
      Exit;
    end;

  // nastavi cernou barvu jako pruhlednou
  DDSetColorKey(FDDSOne, RGB(0, 0, 0));

  // OK:
  Result := True;
end;

function TCarForm.RestoreAll: hResult;
var hRet: hResult;
begin
  hRet := FDDSPrimary._Restore;
  if hRet = DD_OK then
  begin
    hRet := FDDSOne._Restore;
    if hRet = DD_OK then hRet := DDReLoadBitmap(FDDSOne, szBitmap);
  end;
  Result := hRet;
end;

function TCarForm.UpdateFrame: Boolean;
var i, x, y, dx, dy: Integer;
    SourRect, DestRect: TRect;
    hRet: hResult;
    p: TPoint;
begin
  Result := False;

  if FAngle = 0 then
  begin
    DestRect := Rect(0, 0, 319, 199);
    SourRect := DestRect;
    p := ClientToScreen(Point(0, 0));
    OffsetRect(DestRect, p.x, p.y);
    while True do
    begin
      hRet := FDDSPrimary.Blt(@DestRect, FDDSOne, @SourRect, 0, nil);
      if hRet = DD_OK then Break;
      if hRet = DDERR_SURFACELOST then
      begin
        hRet := RestoreAll;
        if hRet <> DD_OK then Exit;
      end;
      if hRet <> DDERR_WASSTILLDRAWING then Exit;
    end;
  end;

  // dame do backsurf. pozadi a ignorujeme neviditelnou barvu
{  SourRect := Rect(0, 0, 319, 199);
  p := ClientToScreen(Point(0, 0));
  DestRect := Rect(p.x, p.y, p.x + 320, p.y + 200);}
  i := (FAngle mod 360) div 5;
  x := i mod 12;
  y := i div 12;
  x := 149 + Round(-Cos(-(FAngle - 1) * Pi / 180) * 70);
  y := 89 + Round(Sin(-(FAngle - 1) * Pi / 180) * 70);
  DestRect := Rect(x, y, x + 19, y + 19);
  SourRect := DestRect;
  p := ClientToScreen(Point(0, 0));
  OffsetRect(DestRect, p.x, p.y);
  while True do
  begin
    hRet := FDDSPrimary.Blt(@DestRect, FDDSOne, @SourRect, 0, nil);
    if hRet = DD_OK then Break;
    if hRet = DDERR_SURFACELOST then
    begin
      hRet := RestoreAll;
      if hRet <> DD_OK then Exit;
    end;
    if hRet <> DDERR_WASSTILLDRAWING then Exit;
  end;

  // vybereme auticko podle uhlu a dame ho na misto
  i := (FAngle mod 360) div 5;
  x := i mod 12;
  y := i div 12;
  SourRect := Rect(x * 20, 200 + y * 20, x * 20 + 19, 200 + y * 20 + 19);
  x := 149 + Round(-Cos(-FAngle * Pi / 180) * 70);
  y := 89 + Round(Sin(-FAngle * Pi / 180) * 70);
  DestRect := Rect(x, y, x + 19, y + 19);
  p := ClientToScreen(Point(0, 0));
  OffsetRect(DestRect, p.x, p.y);
  while True do
  begin
    hRet := FDDSPrimary.Blt(@DestRect, FDDSOne, @SourRect, DDBLT_KEYSRC, nil);
    if hRet = DD_OK then Break;
    if hRet = DDERR_SURFACELOST then
    begin
      hRet := RestoreAll;
      if hRet <> DD_OK then Exit;
    end;
    if hRet <> DDERR_WASSTILLDRAWING then Exit;
  end;

  // OK
  FAngle := FAngle + 1;
  if FAngle >= 360 then FAngle := 0;
  Result := True;
end;

procedure TCarForm.FormActivate(Sender: TObject);
begin
  FActive := True;
end;

procedure TCarForm.FormDeactivate(Sender: TObject);
begin
  FActive := False;
end;

procedure TCarForm.FormShow(Sender: TObject);
begin
  if not InitDirectDraw then Application.Terminate;
end;

procedure TCarForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ExitDirectDraw;
  Application.Minimize;
end;

procedure TCarForm.FormCreate(Sender: TObject);
begin
  FPhase := False; FAngle := 0;
  DXTimer.Enabled := True;
end;

procedure TCarForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) or (Key = VK_F12) then Close;
end;

procedure TCarForm.DXTimerTimer(Sender: TObject);
begin
  if FActive then
    if not UpdateFrame then Close;
end;

end.
